@@ -2,7 +2,7 @@
 #
 # Prepare the stage...
 #
-# $Id: sadi-install.pl,v 1.3 2009-09-18 16:16:41 ubuntu Exp $
+# $Id: sadi-install.pl,v 1.7 2010-03-09 16:39:14 ubuntu Exp $
 # Contact: Edward Kawas <edward.kawas+sadi@gmail.com>
 # -----------------------------------------------------------
 
@@ -189,6 +189,9 @@ eval {
 		mkdir($dir)
 		  || die( "Error creating cgi scripts directory '" . $dir . "':\n$!" );
 	}
+	open (FHO,">$sadi_home/cgi/README") 
+	   and print FHO 'This directory contains entry scripts to your generated SADI services.';
+    close(FHO);
 };
 say $@ ? $@ : "Created install directory '$sadi_home/cgi'.";
 
@@ -203,6 +206,9 @@ eval {
 		  || die(
 				"Error creating installation directory directory '" . $dir . "':\n$!" );
 	}
+	open (FHO,">$samples_home/README") 
+       and print FHO 'This directory is for resources that your SADI service utilizes.';
+    close(FHO);
 };
 say $@ ? $@ : "Created sample-resources directory '$samples_home'.";
 
@@ -217,6 +223,9 @@ eval {
 		mkdir($dir)
 		  || die( "Error creating service definitions directory '" . $dir . "':\n$!" );
 	}
+	open (FHO,">$sadi_home/definitions/README") 
+       and print FHO 'This directory contains the definitions for your SADI services.';
+    close(FHO);
 };
 say $@ ? $@ : "Created service defintions directory '$sadi_home/definitions'.";
 
@@ -231,9 +240,48 @@ eval {
         mkdir($dir)
           || die( "Error creating service async directory '" . $dir . "':\n$!" );
     }
-    chmod 0666, $async_dir;
+    chmod 0777, $async_dir;
+    open (FHO,">$async_dir/README") 
+       and print FHO 'This directory contains the temporary files needed to run asynchronous SADI services.';
+    close(FHO);
 };
-say $@ ? $@ : "Created service async directory '$sadi_home/async'.";
+say $@ ? $@ : "Created service async directory '$async_dir'.";
+
+# create unit test directory if necessary
+my $unittest_dir = $SADICFG::UNITTEST_DIR || "$sadi_home/unittest";
+eval {
+    my ( $v, $d, $f ) = File::Spec->splitpath( $unittest_dir );
+    my $dir = File::Spec->catdir($v);
+    foreach my $part ( File::Spec->splitdir( ( $d . $f ) ) ) {
+        $dir = File::Spec->catdir( $dir, $part );
+        next if -d $dir or -e $dir;
+        mkdir($dir)
+          || die( "Error creating service unit test directory '" . $dir . "':\n$!" );
+    }
+    chmod 0777, $unittest_dir;
+    open (FHO,">$unittest_dir/README") 
+       and print FHO 'This directory contains unit tests for your SADI services.';
+    close(FHO);
+};
+say $@ ? $@ : "Created unit test directory '$unittest_dir'.";
+
+# create directory for xml that user can store input/output in, if necessary
+my $xmlin_dir = "$sadi_home/xml";
+eval {
+    my ( $v, $d, $f ) = File::Spec->splitpath( $xmlin_dir );
+    my $dir = File::Spec->catdir($v);
+    foreach my $part ( File::Spec->splitdir( ( $d . $f ) ) ) {
+        $dir = File::Spec->catdir( $dir, $part );
+        next if -d $dir or -e $dir;
+        mkdir($dir)
+          || die( "Error creating xml directory '" . $dir . "':\n$!" );
+    }
+    chmod 0777, $unittest_dir;
+    open (FHO,">$xmlin_dir/README") 
+       and print FHO 'This directory is a place to put your example in/outputs for your SADI services.';
+    close(FHO);
+};
+say $@ ? $@ : "Created example xml directory '$xmlin_dir'.";
 
 # log files (create, or just change their write permissions)
 my $log_file1 = $SADICFG::LOG_FILE
@@ -276,6 +324,35 @@ my $generated_dir = $SADICFG::GENERATORS_OUTDIR
 my $services_dir = $SADICFG::GENERATORS_IMPL_OUTDIR
   || "$sadi_home/services";
 
+eval {
+    my ( $v, $d, $f ) = File::Spec->splitpath( $generated_dir );
+    my $dir = File::Spec->catdir($v);
+    foreach my $part ( File::Spec->splitdir( ( $d . $f ) ) ) {
+        $dir = File::Spec->catdir( $dir, $part );
+        next if -d $dir or -e $dir;
+        mkdir($dir)
+          || die( "Error creating generated_dir directory '" . $dir . "':\n$!" );
+    }
+    open (FHO,">$generated_dir/README") 
+       and print FHO 'This directory will contain any generated OWL2Perl modules or base SADI service modules.';
+    close(FHO);
+};
+say $@ ? $@ : "Created service defintions directory '$generated_dir'.";
+
+eval {
+    my ( $v, $d, $f ) = File::Spec->splitpath( $services_dir );
+    my $dir = File::Spec->catdir($v);
+    foreach my $part ( File::Spec->splitdir( ( $d . $f ) ) ) {
+        $dir = File::Spec->catdir( $dir, $part );
+        next if -d $dir or -e $dir;
+        mkdir($dir)
+          || die( "Error creating service implementation directory '" . $dir . "':\n$!" );
+    }
+    open (FHO,">$services_dir/README") 
+       and print FHO 'This directory contains your SADI service implementation modules.';
+    close(FHO);
+};
+say $@ ? $@ : "Created service implementation directory '$services_dir'.";
 
 # configuration file (will be found and used, or created)
 my $config_file =
@@ -295,7 +372,8 @@ if ( -e $config_file and !$opt_F ) {
 						   '@GENERATED_DIR@'    => $generated_dir,
 						   '@SERVICES_DIR@'     => $services_dir,
 						   '@DEFINITIONS_DIR@'  => $definitions_dir,
-						   '@ASYNC_OUTDIR@'     => $async_dir, 
+						   '@ASYNC_OUTDIR@'     => $async_dir,
+						   '@UNIT_TEST_DIR@'   => $unittest_dir,
 						   '@HOME_DIR@'         => $sadi_home,
 						   '@LOG4PERL_FILE@'    => $log4perl_file,
 						   '@LOGFILE@'          => $log_file1,
