@@ -3,7 +3,7 @@
 # Author: Edward Kawas <edward.kawas@gmail.com>
 # For copyright and disclaimer see below.
 #
-# $Id: ServiceBase.pm,v 1.7 2009-12-18 19:49:23 ubuntu Exp $
+# $Id: ServiceBase.pm,v 1.8 2010-01-07 21:58:21 ubuntu Exp $
 #-----------------------------------------------------------------
 
 package SADI::Service::ServiceBase;
@@ -15,7 +15,7 @@ use strict;
 
 # add versioning to this module
 use vars qw /$VERSION/;
-$VERSION = sprintf "%d.%02d", q$Revision: 1.7 $ =~ /: (\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.9 $ =~ /: (\d+)\.(\d+)/;
 
 #-----------------------------------------------------------------
 # process_it
@@ -46,7 +46,7 @@ sub as_uni_string {
 #-----------------------------------------------------------------
 
 my @ENV_TO_REPORT =
-  ( 'REMOTE_ADDR', 'REQUEST_URI' ,'HTTP_USER_AGENT', 'CONTENT_LENGTH', 'CONTENT_TYPE' );
+  ( 'REMOTE_ADDR', 'REQUEST_URI' ,'HTTP_USER_AGENT', 'CONTENT_LENGTH', 'CONTENT_TYPE', 'HTTP_ACCEPT' );
 
 sub log_request {
 	my ($self) = shift;
@@ -74,6 +74,30 @@ sub get_service_signature {
 	$self->throw("Couldn't find a signature for '$name'!.");
 }
 
+# returns the request content type
+# defaults to application/rdf+xml
+sub get_request_content_type {
+	my ($self) = @_;
+    my $CONTENT_TYPE = 'application/rdf+xml';
+    if (defined $ENV{CONTENT_TYPE}) {
+        $CONTENT_TYPE = 'text/rdf+n3' if $ENV{CONTENT_TYPE} =~ m|text/rdf\+n3|gi;
+        $CONTENT_TYPE = 'text/rdf+n3' if $ENV{CONTENT_TYPE} =~ m|text/n3|gi;
+    }
+    return $CONTENT_TYPE;
+}
+
+# returns the response requested content type
+# defaults to application/rdf+xml
+sub get_response_content_type {
+    my ($self) = @_;
+    my $CONTENT_TYPE = 'application/rdf+xml';
+    if (defined $ENV{HTTP_ACCEPT}) {
+        $CONTENT_TYPE = 'text/rdf+n3' if $ENV{HTTP_ACCEPT} =~ m|text/rdf\+n3|gi;
+        $CONTENT_TYPE = 'text/rdf+n3' if $ENV{HTTP_ACCEPT} =~ m|text/n3|gi;
+    }
+    return $CONTENT_TYPE;
+}
+
 1;
 __END__
 
@@ -84,9 +108,11 @@ SADI::ServiceBase - a super-class for all SADI services
 =head1 SYNOPSIS
 
  use base qw( SADI::ServiceBase )
- 
+
 =head1 DESCRIPTION
 
+A super class for all SADI services implemented with SADISeS.
+
 =head1 SUBROUTINES
 
 =head2 process_it
@@ -118,6 +144,18 @@ SADI string (type SADI::Data::String).
  # information about the current call (request) that can be used in a
  # log entry
 
+=head2 get_request_content_type
+
+ # Returns the content type of the incoming data, defaults to application/rdf+xml.
+ #
+ # Possible values: 'application/rdf+xml', 'text/rdf+n3'
+
+=head2 get_response_content_type
+
+ # Returns the requested content type of the outgoing data, defaults to application/rdf+xml.
+ #
+ # Possible values: 'application/rdf+xml', 'text/rdf+n3'
+
 =head1 AUTHORS, COPYRIGHT, DISCLAIMER
 
  Edward Kawas  (edward.kawas [at] gmail [dot] com)
